#!/usr/bin/env node
"use strict";
var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getOwnPropSymbols = Object.getOwnPropertySymbols;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __propIsEnum = Object.prototype.propertyIsEnumerable;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues = (a, b) => {
  for (var prop in b || (b = {}))
    if (__hasOwnProp.call(b, prop))
      __defNormalProp(a, prop, b[prop]);
  if (__getOwnPropSymbols)
    for (var prop of __getOwnPropSymbols(b)) {
      if (__propIsEnum.call(b, prop))
        __defNormalProp(a, prop, b[prop]);
    }
  return a;
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(
  // If the importer is in node compatibility mode or this is not an ESM
  // file that has been converted to a CommonJS file using a Babel-
  // compatible transform (i.e. "__esModule" has not been set), then set
  // "default" to the CommonJS "module.exports" for node compatibility.
  isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target,
  mod
));
var __async = (__this, __arguments, generator) => {
  return new Promise((resolve2, reject) => {
    var fulfilled = (value) => {
      try {
        step(generator.next(value));
      } catch (e) {
        reject(e);
      }
    };
    var rejected = (value) => {
      try {
        step(generator.throw(value));
      } catch (e) {
        reject(e);
      }
    };
    var step = (x) => x.done ? resolve2(x.value) : Promise.resolve(x.value).then(fulfilled, rejected);
    step((generator = generator.apply(__this, __arguments)).next());
  });
};

// src/index.ts
var import_json_schema_to_typescript2 = require("json-schema-to-typescript");
var import_fs = __toESM(require("fs"));
var import_camelcase = __toESM(require("camelcase"));

// src/defaultCustomMapper.ts
function defaultCustomMapper(key, obj) {
  switch (obj.field_type) {
    case "seo-metatags":
      return {
        [key]: {
          type: "object",
          properties: {
            _uid: {
              type: "string"
            },
            title: {
              type: "string"
            },
            plugin: {
              type: "string"
            },
            og_image: {
              type: "string"
            },
            og_title: {
              type: "string"
            },
            description: {
              type: "string"
            },
            twitter_image: {
              type: "string"
            },
            twitter_title: {
              type: "string"
            },
            og_description: {
              type: "string"
            },
            twitter_description: {
              type: "string"
            }
          }
        }
      };
    default:
      return {};
  }
}

// src/genericTypes.ts
var import_json_schema_to_typescript = require("json-schema-to-typescript");
var typeFuncs = {
  "asset": generateAssetTypeIfNotYetGenerated,
  "multiasset": generateMultiAssetTypeIfNotYetGenerated,
  "multilink": generateMultiLinkTypeIfNotYetGenerated,
  "table": generateTableTypeIfNotYetGenerated,
  "richtext": generateRichtextTypeIfNotYetGenerated
};
var toGenerateWhitelist = Object.keys(typeFuncs);
function compileType(obj, name, compilerOptions) {
  return __async(this, null, function* () {
    const ts = yield (0, import_json_schema_to_typescript.compile)(obj, name, compilerOptions);
    toGenerateWhitelist.splice(toGenerateWhitelist.indexOf(name), 1);
    return ts;
  });
}
function generate(type, title, compilerOptions) {
  return __async(this, null, function* () {
    return yield typeFuncs[type](title, compilerOptions);
  });
}
var TYPES = Object.keys(typeFuncs);
function generateAssetTypeIfNotYetGenerated(title, compilerOptions) {
  return __async(this, null, function* () {
    if (!toGenerateWhitelist.includes("asset"))
      return;
    const obj = {
      "$id": "#/asset",
      title,
      type: "object",
      required: ["id", "filename", "name"],
      properties: {
        alt: {
          type: "string"
        },
        copyright: {
          type: "string"
        },
        id: {
          type: "number"
        },
        filename: {
          type: "string"
        },
        name: {
          type: "string"
        },
        title: {
          type: "string"
        },
        focus: {
          type: "string"
        }
      }
    };
    try {
      return yield compileType(obj, "asset", compilerOptions);
    } catch (e) {
      console.log("ERROR", e);
    }
  });
}
function generateRichtextTypeIfNotYetGenerated(title, compilerOptions) {
  return __async(this, null, function* () {
    if (!toGenerateWhitelist.includes("richtext"))
      return;
    const obj = {
      "$id": "#/richtext",
      title,
      type: "object",
      required: ["type"],
      properties: {
        type: {
          type: "string"
        },
        content: {
          type: "array",
          items: {
            "$ref": "#"
          }
        },
        marks: {
          type: "array",
          items: {
            "$ref": "#"
          }
        },
        attrs: {},
        text: {
          type: "string"
        }
      }
    };
    try {
      return yield compileType(obj, "richtext", compilerOptions);
    } catch (e) {
      console.log("ERROR", e);
    }
  });
}
function generateMultiAssetTypeIfNotYetGenerated(title, compilerOptions) {
  return __async(this, null, function* () {
    if (!toGenerateWhitelist.includes("multiasset"))
      return;
    const obj = {
      "$id": "#/multiasset",
      title,
      type: "array",
      items: {
        type: "object",
        required: ["id", "filename", "name"],
        properties: {
          alt: {
            type: "string"
          },
          copyright: {
            type: "string"
          },
          id: {
            type: "number"
          },
          filename: {
            type: "string"
          },
          name: {
            type: "string"
          },
          title: {
            type: "string"
          }
        }
      }
    };
    try {
      return yield compileType(obj, "multiasset", compilerOptions);
    } catch (e) {
      console.log("ERROR", e);
    }
  });
}
function generateMultiLinkTypeIfNotYetGenerated(title, compilerOptions) {
  return __async(this, null, function* () {
    if (!toGenerateWhitelist.includes("multilink"))
      return;
    const obj = {
      $id: "#/multilink",
      title,
      "oneOf": [
        {
          type: "object",
          properties: {
            cached_url: {
              type: "string"
            },
            linktype: {
              type: "string"
            }
          }
        },
        {
          type: "object",
          properties: {
            id: {
              type: "string"
            },
            cached_url: {
              type: "string"
            },
            anchor: {
              type: "string"
            },
            linktype: {
              type: "string",
              enum: ["story"]
            },
            story: {
              type: "object",
              required: ["name", "id", "uuid", "slug", "full_slug"],
              properties: {
                name: {
                  type: "string"
                },
                created_at: {
                  type: "string",
                  format: "date-time"
                },
                published_at: {
                  type: "string",
                  format: "date-time"
                },
                id: {
                  type: "integer"
                },
                uuid: {
                  type: "string",
                  format: "uuid"
                },
                content: {
                  type: "object"
                },
                slug: {
                  type: "string"
                },
                full_slug: {
                  type: "string"
                },
                sort_by_date: {
                  type: ["null", "string"],
                  format: "date-time"
                },
                position: {
                  type: "integer"
                },
                tag_list: {
                  type: "array",
                  items: {
                    type: "string"
                  }
                },
                is_startpage: {
                  type: "boolean"
                },
                parent_id: {
                  type: ["null", "integer"]
                },
                meta_data: {
                  type: ["null", "object"]
                },
                group_id: {
                  type: "string",
                  format: "uuid"
                },
                first_published_at: {
                  type: "string",
                  format: "date-time"
                },
                release_id: {
                  type: ["null", "integer"]
                },
                lang: {
                  type: "string"
                },
                path: {
                  type: ["null", "string"]
                },
                alternates: {
                  type: "array"
                },
                default_full_slug: {
                  type: ["null", "string"]
                },
                translated_slugs: {
                  type: ["null", "array"]
                }
              }
            }
          }
        },
        {
          type: "object",
          properties: {
            url: {
              type: "string"
            },
            cached_url: {
              type: "string"
            },
            anchor: {
              type: "string"
            },
            linktype: {
              type: "string",
              enum: ["asset", "url"]
            }
          }
        },
        {
          type: "object",
          properties: {
            email: {
              type: "string"
            },
            linktype: {
              type: "string",
              enum: ["email"]
            }
          }
        }
      ]
    };
    try {
      return yield compileType(obj, "multilink", compilerOptions);
    } catch (e) {
      console.log("ERROR", e);
    }
  });
}
function generateTableTypeIfNotYetGenerated(title, compilerOptions) {
  return __async(this, null, function* () {
    if (!toGenerateWhitelist.includes("table"))
      return;
    const obj = {
      $id: "#/table",
      title,
      type: "object",
      required: ["tbody", "thead"],
      properties: {
        thead: {
          type: "array",
          items: {
            type: "object",
            required: ["_uid", "component"],
            properties: {
              _uid: {
                type: "string"
              },
              value: {
                type: "string"
              },
              component: {
                type: "number"
              }
            }
          }
        },
        tbody: {
          type: "array",
          items: {
            type: "object",
            required: ["_uid", "component", "body"],
            properties: {
              _uid: {
                type: "string"
              },
              body: {
                type: "array",
                items: {
                  type: "object",
                  properties: {
                    _uid: {
                      type: "string"
                    },
                    value: {
                      type: "string"
                    },
                    component: {
                      type: "number"
                    }
                  }
                }
              },
              component: {
                type: "number"
              }
            }
          }
        }
      }
    };
    try {
      return yield compileType(obj, "table", compilerOptions);
    } catch (e) {
      console.log("ERROR", e);
    }
  });
}

// src/index.ts
function storyblokToTypescript(_0) {
  return __async(this, arguments, function* ({
    componentsJson = { components: [] },
    compilerOptions = {},
    customTypeParser,
    path = "src/typings/generated/components-schema.ts",
    titleSuffix = "_storyblok",
    titlePrefix = ""
  }) {
    compilerOptions = __spreadValues({
      unknownAny: false,
      bannerComment: "",
      unreachableDefinitions: true
    }, compilerOptions);
    const tsString = [`import {StoryblokStory} from 'storyblok-generate-ts'`, ``];
    const getTitle = (t) => titlePrefix + t + titleSuffix;
    const getStoryTypeTitle = (t) => `StoryblokStory<${(0, import_camelcase.default)(getTitle(t), { pascalCase: true })}>`;
    const groupUuids = {};
    componentsJson.components.forEach((value) => {
      if (value.component_group_uuid) {
        if (!groupUuids[value.component_group_uuid]) {
          groupUuids[value.component_group_uuid] = [];
        }
        groupUuids[value.component_group_uuid].push((0, import_camelcase.default)(getTitle(value.name), {
          pascalCase: true
        }));
      }
    });
    function genTsSchema() {
      return __async(this, null, function* () {
        for (const values of componentsJson.components) {
          const obj = {
            "$id": "#/" + values.name,
            title: getTitle(values.name),
            type: "object"
          };
          obj.properties = yield typeMapper(values.schema, obj.title);
          obj.properties._uid = {
            type: "string"
          };
          obj.properties.component = {
            type: "string",
            enum: [values.name]
          };
          if (values.name === "global" || values.name === "page") {
            obj.properties.uuid = {
              type: "string"
            };
          }
          const requiredFields = ["_uid", "component"];
          Object.keys(values.schema).forEach((key) => {
            if (values.schema[key].required) {
              requiredFields.push(key);
            }
          });
          if (requiredFields.length) {
            obj.required = requiredFields;
          }
          try {
            const ts = yield (0, import_json_schema_to_typescript2.compile)(obj, values.name, compilerOptions);
            tsString.push(ts);
          } catch (e) {
            console.log("ERROR", e);
          }
        }
      });
    }
    function typeMapper() {
      return __async(this, arguments, function* (schema = {}, title) {
        const parseObj = {};
        for (const key of Object.keys(schema)) {
          if (key.startsWith("tab-")) {
            continue;
          }
          const obj = {};
          const schemaElement = schema[key];
          const type = schemaElement.type;
          if (TYPES.includes(type)) {
            const ts = yield generate(type, getTitle(type), compilerOptions);
            if (ts) {
              tsString.push(ts);
            }
          } else if (type === "custom") {
            Object.assign(parseObj, defaultCustomMapper(key, schemaElement));
            if (typeof customTypeParser === "function") {
              Object.assign(parseObj, customTypeParser(key, schemaElement));
            }
            continue;
          }
          const element = parseSchema(schemaElement);
          if (!element) {
            continue;
          }
          obj[key] = element;
          if (TYPES.includes(type)) {
            obj[key].tsType = (0, import_camelcase.default)(getTitle(type), {
              pascalCase: true
            });
          } else if (type === "bloks") {
            if (schemaElement.restrict_components) {
              if (schemaElement.restrict_type === "groups") {
                if (Array.isArray(schemaElement.component_group_whitelist) && schemaElement.component_group_whitelist.length) {
                  let currentGroupElements = [];
                  schemaElement.component_group_whitelist.forEach((groupId) => {
                    const currentGroup = groupUuids[groupId];
                    if (Array.isArray(currentGroup)) {
                      currentGroupElements = [...currentGroupElements, ...currentGroup];
                    } else {
                      console.log("Group has no members: ", groupId);
                    }
                  });
                  if (currentGroupElements.length == 0) {
                    obj[key].tsType = `never[]`;
                  } else {
                    obj[key].tsType = `(${currentGroupElements.join(" | ")})[]`;
                  }
                }
              } else {
                if (Array.isArray(schemaElement.component_whitelist) && schemaElement.component_whitelist.length) {
                  obj[key].tsType = `(${schemaElement.component_whitelist.map((i) => (0, import_camelcase.default)(getTitle(i), {
                    pascalCase: true
                  })).join(" | ")})[]`;
                } else {
                  console.log("No whitelisted component found");
                }
              }
            } else {
              console.log("Type: bloks array but not whitelisted (will result in all elements):", title);
            }
          }
          Object.assign(parseObj, obj);
        }
        return parseObj;
      });
    }
    function parseSchema(element) {
      if (TYPES.includes(element.type)) {
        return {
          type: element.type
        };
      }
      let type = "any";
      let options = [];
      if (Array.isArray(element.options) && element.options.length) {
        options = element.options.map((item) => item.value);
      }
      if (options.length && element.exclude_empty_option !== true) {
        options.unshift("");
      }
      if (!element.source && element.options !== void 0) {
        type = "string";
      }
      if (element.source === "internal_stories" && element.filter_content_type === void 0) {
        type = "any";
      }
      if (element.source === "internal_stories" && element.filter_content_type) {
        if (element.type === "option") {
          if (Array.isArray(element.filter_content_type)) {
            return {
              tsType: `(${element.filter_content_type.map((type2) => getStoryTypeTitle(type2)).join(" | ")} | string )`
            };
          } else {
            return {
              tsType: `(${getStoryTypeTitle(element.filter_content_type)} | string )`
            };
          }
        }
        if (element.type === "options") {
          if (Array.isArray(element.filter_content_type)) {
            return {
              tsType: `(${element.filter_content_type.map((type2) => getStoryTypeTitle(type2)).join(" | ")} | string )[]`
            };
          } else {
            return {
              tsType: `(${getStoryTypeTitle(element.filter_content_type)} | string )[]`
            };
          }
        }
      }
      if (element.source === "internal_languages") {
        type = "string";
      }
      if (element.source === "internal") {
        type = ["number", "string"];
      }
      if (element.source === "external") {
        type = "string";
      }
      if (element.type === "option") {
        if (options.length) {
          return {
            type,
            enum: options
          };
        }
        return {
          type
        };
      }
      if (element.type === "options") {
        if (options.length) {
          return {
            type: "array",
            items: {
              enum: options
            }
          };
        }
        return {
          type: "array",
          items: { type }
        };
      }
      switch (element.type) {
        case "text":
          return { type: "string" };
        case "bloks":
          return { type: "array" };
        case "number":
          return { type: "string" };
        case "image":
          return { type: "string" };
        case "boolean":
          return { type: "boolean" };
        case "textarea":
          return { type: "string" };
        case "markdown":
          return { type: "string" };
        case "datetime":
          return { type: "string" };
        default:
          return { type: "any" };
      }
    }
    yield genTsSchema();
    if (path) {
      import_fs.default.writeFileSync(path, tsString.join("\n"));
    }
    return tsString;
  });
}

// src/cli.ts
var import_path = require("path");
var fs2 = __toESM(require("fs"));
var [, , ...args] = process.argv;
var parseValue = (value) => {
  if (value.match(/^(true|false)$/)) {
    return value === "true";
  }
  if (value.match(/^[\d.]+$/)) {
    return Number(value);
  }
  return value;
};
var getDataFromPath = (path) => __async(exports, null, function* () {
  if (!path) {
    return {};
  }
  const sources = path.split(",");
  const isList = sources.length > 1;
  try {
    if (!isList)
      return JSON.parse(fs2.readFileSync(sources[0], "utf8"));
    const data = [];
    sources.forEach((source) => {
      data.push(JSON.parse(fs2.readFileSync(source, "utf8")));
    });
    return data;
  } catch (err) {
    console.error(`Can not load json file from ${path}`);
    return Promise.reject(err);
  }
});
var createContentList = (content, key) => {
  if (content[key])
    return content[key];
  else if (Array.isArray(content))
    return [...content];
  else
    return [content];
};
var props = {};
args.forEach((key) => {
  const [name, value] = key.split("=").map((i) => i.trim());
  if (name.match(/\w+\.\w+/)) {
    const [objectName, objectProperty] = name.split(".");
    props[objectName] = props[objectName] || {};
    props[objectName][objectProperty] = parseValue(value);
  } else {
    props[name] = parseValue(value);
  }
});
if (!props.source) {
  console.log("there is no source path to components file");
  process.exit();
}
if (props.target && !props.target.endsWith(".ts")) {
  props.target += ".d.ts";
}
getDataFromPath(props.source).then((rawComponents) => {
  const components = createContentList(rawComponents, "components");
  const options = {
    componentsJson: { components },
    compilerOptions: props.compilerOptions || {},
    path: (0, import_path.resolve)(props.target || "./storyblok-component-types.d.ts")
  };
  if (props.titlePrefix !== void 0) {
    options.titlePrefix = props.titlePrefix;
  }
  if (props.titleSuffix !== void 0) {
    options.titleSuffix = props.titleSuffix;
  }
  if (props.customTypeParser) {
    options.customTypeParser = require((0, import_path.resolve)(props.customTypeParser));
  }
  storyblokToTypescript(options);
}).catch((e) => {
  console.log("Something went wrong while generating your types:");
  console.log(e);
});
