import { Options } from 'json-schema-to-typescript';
import { JSONSchema4 } from 'json-schema';

type CompilerOptions = Partial<Options>;
interface StoryblokTsOptions {
    componentsJson: {
        components: JSONSchema4[];
    };
    customTypeParser?: (key: string, options: JSONSchema4) => void;
    compilerOptions?: CompilerOptions;
    path?: string;
    titleSuffix?: string;
    titlePrefix?: string;
}
interface StoryblokStory<TContent> {
    name: string;
    created_at: string;
    published_at: string;
    id: number;
    uuid: string;
    content: TContent;
    slug: string;
    full_slug: string;
    sort_by_date: string | null;
    position: number;
    tag_list: string[];
    is_startpage: boolean;
    parent_id: number;
    meta_data: any;
    group_id: string;
    first_published_at: string | null;
    release_id?: number | null;
    lang: string;
    path?: string;
    alternates: {
        id: number;
        name: string;
        slug: string;
        published: boolean;
        full_slug: string;
        is_folder: boolean;
        parent_id: number;
    }[];
    default_full_slug: string;
    translated_slugs: {
        path: string;
        name: string | null;
        lang: string;
    }[];
    _stopResolving?: boolean;
}

declare function storyblokToTypescript({ componentsJson, compilerOptions, customTypeParser, path, titleSuffix, titlePrefix }: StoryblokTsOptions): Promise<string[]>;

export { StoryblokStory, storyblokToTypescript as default };
